package org.monaka.book.android.camera;

import java.io.IOException;

import android.content.Context;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;

public class CameraPreview extends SurfaceView implements Callback {
    private SurfaceHolder holder;
    protected Camera camera;

	public CameraPreview(Context context, AttributeSet attributeSet) {
		super(context, attributeSet);
		holder = getHolder();
		holder.addCallback(this);
		holder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}

	@Override
	public void surfaceChanged(SurfaceHolder surfaceHolder, int format, int width, int height) {
		Camera.Parameters parameters = camera.getParameters();
		Camera.Size previewSize = parameters.getSupportedPreviewSizes().get(0);
		parameters.setPreviewSize(previewSize.width, previewSize.height);
		parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
		camera.setParameters(parameters);
		camera.startPreview();
	}

	@Override
	public void surfaceCreated(SurfaceHolder surfaceHolder) {
		if (camera != null) { // to make sure.
			return;
		}

        camera = Camera.open();

        try {
			camera.setPreviewDisplay(surfaceHolder);
		} catch (IOException e) {
			camera.release();
			camera = null;
		}
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder surfaceHolder) {
		camera.stopPreview();
		camera.release();
		camera = null;
	}

}
