package org.monaka.book.android.camera;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.Intent;
import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

public class WaitingActivity extends Activity {
	private static final String TAG = null;
	private UsbManager usbManager;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.waiting);

		usbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
	}

	@Override
	public void onResume() {
		super.onResume();

		Intent intent = getIntent();
		Log.d(TAG, "intent: " + intent);
		String action = intent.getAction();

		UsbDevice device = (UsbDevice) intent
				.getParcelableExtra(UsbManager.EXTRA_DEVICE);
		if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
			setDevice(device);
		} else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
			if (device != null && device.equals(device)) {
				setDevice(null);
			}
		}
	}

	private static int TIMEOUT = 5000;
	private boolean forceClaim = true;

	private void setDevice(UsbDevice device) {

		FelicaWaiting waiting = new FelicaWaiting(getApplicationContext());
		waiting.execute(device);
	}

	class FelicaWaiting extends AsyncTask<UsbDevice, Void, Void> implements
			OnCancelListener {

		private UsbDeviceConnection connection;
		private UsbEndpoint epIn;
		private UsbEndpoint epOut;
		private Context context;

		public FelicaWaiting(Context context) {
			this.context = context;
		}

		private byte[] buildPacket(byte[] param) {
			byte[] packet = new byte[param.length + 7];
			packet[0] = 0;
			packet[1] = 0;
			packet[2] = (byte) 0xff;
			packet[3] = (byte) param.length;
			packet[4] = (byte) (~param.length + 1);
			System.arraycopy(param, 0, packet, 5, param.length);

			int mod = 0;
			for (byte b : param) {
				mod += b;
			}
			packet[5 + param.length] = (byte) (~mod + 1);
			packet[6 + param.length] = 0;

			return packet;
		}

		private byte[] communicate(byte[] packet) throws PasoriException {
			connection.bulkTransfer(epOut, packet, packet.length, TIMEOUT);

			byte[] response = new byte[256];
			connection.bulkTransfer(epIn, response, response.length, TIMEOUT);
			assertAck(response);

			int len = 0;
			do {
				len = connection.bulkTransfer(epIn, response, response.length, TIMEOUT);
			} while (len < 0);

			return response;
		}

		private void s330_rf_antenna(boolean antenna_on) throws PasoriException {
			byte[] rf_antenna = { (byte)0xd4, 0x32, 0x01, (byte)0xff };
			if (antenna_on) {
				rf_antenna[3] = 0x01;
			}

			byte[] packet = buildPacket(rf_antenna);
			communicate(packet);
		}

		private void s330_polling()  throws PasoriException {
			byte[] command = { (byte)0xd4, 0x4a, 0x01, 0x01, 0x00, (byte)0xff, (byte)0xff, 0x00, 0x00 };

			byte[] packet = buildPacket(command);
			communicate(packet);
		}

		private void assertAck(byte[] response) throws PasoriException {
			if (response[2] == 0x00 && response[3] == 0xff) {
				throw new PasoriException();
			}
		}

		@Override
		protected void onPostExecute(Void unused) {
			Intent intent = new Intent(context, CameraActivity.class);
			startActivity(intent);
		}

		@Override
		protected Void doInBackground(UsbDevice... arg0) {
			UsbDevice device = arg0[0];

			UsbInterface intf = device.getInterface(0);
			for (int i = 0; i < intf.getEndpointCount(); i++) {
				UsbEndpoint ep = intf.getEndpoint(i);
				int direction = ep.getDirection();
				if (direction == UsbConstants.USB_DIR_IN) {
					epIn = ep;
				} else if (direction == UsbConstants.USB_DIR_OUT) {
					epOut = ep;
				}
			}
			if (epIn == null || epOut == null) {
				cancel(true);
			}

			connection = usbManager.openDevice(device);
			connection.claimInterface(intf, forceClaim);

			try {
				s330_rf_antenna(true);
				s330_polling();
			} catch (PasoriException e) {
				Log.e(TAG, "Failed to initialize PaSoRi", e);
				cancel(true);
			}

			return null;
		}

		@Override
		public void onCancel(DialogInterface dialogInterface) {
			Toast.makeText(context, "Internal error (Pasori connection)",
					Toast.LENGTH_LONG).show();
		}
	}

	class PasoriException extends Exception {
		private static final long serialVersionUID = 1L;
	}
}
